#!/usr/bin/env bash

LEIN_BUILD_DIR=${1}
LEIN_CACHE_DIR=${2}
LEIN_DIR="$LEIN_BUILD_DIR/.lein"

# Determine Leiningen version
if is_lein2 $LEIN_BUILD_DIR; then
  LEIN_VERSION="2.6.1"
else
  LEIN_VERSION="1.7.1"
fi

LEIN_JAR_URL="https://lang-jvm.s3.amazonaws.com/leiningen-$LEIN_VERSION-standalone.jar"
LEIN_JAR_CACHE_PATH="$LEIN_CACHE_DIR/leiningen-$LEIN_VERSION-standalone.jar"
LEIN_JAR_SLUG_PATH="$LEIN_DIR/leiningen-$LEIN_VERSION-standalone.jar"

is_lein2() {
  test [ "$(grep ":min-lein-version[[:space:]]\+\"2" $1/project.clj)" != "" ]
}

_install_lein_script() {
  local lein_version=$1
  local lein_bin_src=$2
  local lein_bin_path=$3

  echo "       Writing: lein script"
  mkdir -p $(dirname $lein_bin_path)
  cp $lein_bin_src $lein_bin_path
  sed -i s/##LEIN_VERSION##/$lein_version/ $lein_bin_path
}

_install_lein_jar() {
  if [ ! -r "$LEIN_JAR_CACHE_PATH" ]; then
    echo "-----> Installing Leiningen"
    echo "       Downloading: leiningen-$LEIN_VERSION-standalone.jar"
    mkdir -p $(dirname $LEIN_JAR_CACHE_PATH)
    curl --retry 3 --silent --show-error --max-time 120 -L -o "$LEIN_JAR_CACHE_PATH" $LEIN_JAR_URL
  else
    echo "-----> Using cached Leiningen $LEIN_VERSION"
  fi

  if [ "$LEIN_VERSION" = "1.7.1" ]; then
    echo "       To use Leiningen 2.x, add this to project.clj: :min-lein-version \"2.0.0\""
  fi
}

install_lein() {
  local build_dir=${1:-$LEIN_BUILD_DIR}
  if is_lein2 $build_dir; then
    LEIN_BIN_SOURCE="$(dirname $0)/../opt/lein2"
    if [ "$(grep :uberjar-name $build_dir/project.clj)" != "" ]; then
        LEIN_BUILD_TASK="${LEIN_BUILD_TASK:-uberjar}"
        LEIN_INCLUDE_IN_SLUG="${LEIN_INCLUDE_IN_SLUG:-no}"
    elif [ "$(grep lein-npm $build_dir/project.clj)" != "" ]; then
        LEIN_BUILD_TASK=${LEIN_BUILD_TASK:-"with-profile production do deps, compile :all"}
    else
        LEIN_BUILD_TASK=${LEIN_BUILD_TASK:-"with-profile production compile :all"}
    fi
  else
    LEIN_BIN_SOURCE="$(dirname $0)/../opt/lein1"
    LEIN_BUILD_TASK=${LEIN_BUILD_TASK:-"deps"}
    if [ "$LEIN_DEV" = "" ]; then
      export LEIN_NO_DEV=y
    fi
    RLWRAP=yes
    warning "No :min-lein-version found in project.clj; using $LEIN_VERSION.
You probably don't want this!"
  fi

  _install_lein_jar

  # install rlwrap binary on lein 1.x
  if [ "$RLWRAP" = "yes" ]; then
    RLWRAP_BIN_URL="https://lang-jvm.s3.amazonaws.com/rlwrap-0.3.7"
    RLWRAP_BIN_PATH="$LEIN_DIR/bin/rlwrap"
    echo "       Downloading: rlwrap-0.3.7"
    mkdir -p $(dirname $RLWRAP_BIN_PATH)
    curl --retry 3 --silent --show-error --max-time 60 -L -o $RLWRAP_BIN_PATH $RLWRAP_BIN_URL
    chmod +x $RLWRAP_BIN_PATH
  fi

  _install_lein_script $LEIN_VERSION $LEIN_BIN_SOURCE "$LEIN_DIR/bin/lein"
}
